///////////////////////////////////////////////////////////////////////////////
// luminance.cpp
// =============
// convert color image to gray scale level image
// Luminance = 0.299*Red + 0.587*Green + 0.114*Blue
// Fast approx. = (2*Red + 5*Green + 1*Blue) / 8
//
//  AUTHOR: Song Ho Ahn (song.ahn@gamil.com)
// CREATED: 2006-09-21
// UPDATED: 2023-05-12
///////////////////////////////////////////////////////////////////////////////

#include "luminance.h"



///////////////////////////////////////////////////////////////////////////////
// convert 24 bit BGR colour image to 8 bit grayscale
///////////////////////////////////////////////////////////////////////////////
bool dip::bgr2gray(const unsigned char* in, int width, int height, unsigned char* out, bool fast)
{
    // check NULL pointer
    if(!in || !out)
        return false;

    int size = width * height;  // number of pixels

    if(!fast)   // slow method ****************************
    {
        int i,j;
        for(i = 0, j = 0; i < size; ++i, j+=3)
        {
            out[i] = (unsigned char)(0.114f * in[j] + 0.587f * in[j+1] + 0.299f * in[j+2] + 0.5f);
        }
        return true;
    }

    // fast estimation ************************************
    unsigned int tmp = 0;
    int i = 0;
    while(i++ < size)
    {
        tmp  = *in++;                       // add blue (1 * blue)
        tmp += (*in << 2) + *in++;          // add green (5 * green)
        tmp += *in++ << 1;                  // add red (2 * red)
        *out++ = (unsigned char)(tmp >> 3); // divide by 8
    }

    return true;
}

///////////////////////////////////////////////////////////////////////////////
// convert 32 bit BGRA colour image to 8 bit grayscale
///////////////////////////////////////////////////////////////////////////////
bool dip::bgra2gray(const unsigned char* in, int width, int height, unsigned char* out, bool fast)
{
    // check NULL pointer
    if(!in || !out)
        return false;

    int size = width * height;  // number of pixels

    if(!fast)   // slow methode ***************************
    {
        int i,j;
        for(i = 0, j = 0; i < size; ++i, j+=4)
        {
            out[i] = (unsigned char)(0.114f * in[j] + 0.587f * in[j+1] + 0.299f * in[j+2] + 0.5f);
        }
        return true;
    }

    // fast estimation ************************************
    unsigned int tmp = 0;
    int i = 0;
    while(i++ < size)
    {
        tmp  = *in++;                       // add blue (1 * blue)
        tmp += (*in << 2) + *in++;          // add green (5 * green)
        tmp += *in++ << 1;                  // add red (2 * red)
        ++in;                               // skip alpha
        *out++ = (unsigned char)(tmp >> 3); // divide by 8
    }

    return true;
}





///////////////////////////////////////////////////////////////////////////////
// convert 24 bit RGB colour image to 8 bit grayscale
///////////////////////////////////////////////////////////////////////////////
bool dip::rgb2gray(const unsigned char* in, int width, int height, unsigned char* out, bool fast)
{
    // check NULL pointer
    if(!in || !out)
        return false;

    int size = width * height;  // number of pixels

    if(!fast)   // slow method ****************************
    {
        int i, j;
        for(i = 0, j = 0; i < size; ++i, j+=3)
        {
            out[i] = (unsigned char)(0.229f * in[j] + 0.587f * in[j+1] + 0.114f * in[j+2] + 0.5f);
        }
        return true;
    }

    // fast estimation ************************************
    unsigned int tmp = 0;
    int i = 0;
    while(i++ < size)
    {
        tmp  = *in++ << 1;                  // add red (2 * red)
        tmp += (*in << 2) + *in++;          // add green (5 * green)
        tmp += *in++;                       // add blue (1 * blue)
        *out++ = (unsigned char)(tmp >> 3); // divide by 8
    }

    return true;
}

///////////////////////////////////////////////////////////////////////////////
// convert 32 bit RGBA colour image to 8 bit grayscale
///////////////////////////////////////////////////////////////////////////////
bool dip::rgba2gray(const unsigned char* in, int width, int height, unsigned char* out, bool fast)
{
    // check NULL pointer
    if(!in || !out)
        return false;

    int size = width * height;  // number of pixels

    if(!fast) // slow method ******************************
    {
        int i, j;
        for(i = 0, j = 0; i < size; ++i, j+=4)
        {
            out[i] = (unsigned char)(0.229f * in[j] + 0.587f * in[j+1] + 0.114f * in[j+2] + 0.5f);
        }
        return true;
    }

    // fast estimation ********************************
    unsigned int tmp = 0;
    int i = 0;
    while(i++ < size)
    {
        tmp  = *in++ << 1;                  // add red (2 * red)
        tmp += (*in << 2) + *in++;          // add green (5 * green)
        tmp += *in++;                       // add blue (1 * blue)
        ++in;                               // skip alpha
        *out++ = (unsigned char)(tmp >> 3); // divide by 8
    }

    return true;
}
